@@ -135,6 +135,11 @@
   :group 'yacas
   :type 'integer)
 
+(defcustom yacas-use-emacs-help t
+  "*If non-nil, use browse-url to get help"
+  :group 'yacas
+  :type 'boolean)
+
 ;(defcustom yacas-prompt-regexp "^\\(In> \\|Out> \\)"
 ;  "A regular expression for the Yacas prompt."
 ;  :group 'yacas
@@ -387,7 +392,7 @@ can be used to scroll through the input history.
 
 (defun yacas-help ()
   (interactive)
-  (let ((fn (read-string "Help on: ")))
+  (let ((fn (completing-read "Help on: " yacas-words)))
     (select-window (split-window))
     (if (string= fn "")
         (browse-url
@@ -757,6 +762,8 @@ M-s will bring the next input matching
                    (delete-process inferior-yacas-process))
                (setq inferior-yacas-process nil)
                (run-hooks 'inferior-yacas-exit-hook))) t t)
+  (if yacas-use-emacs-help
+      (yacas-use-emacs-help-start))
   (run-hooks 'inferior-yacas-mode-hook))
 
 ;;;; Interacting with the Yacas process
@@ -850,16 +857,94 @@ M-s will bring the next input matching
         (setq yacas-input-end (point))
         (comint-send-input))
   (while (not (yacas-new-prompt-p))
-    (sit-for 0.100)))
+    (sit-for 0.100))
+  (goto-char (point-max)))
+
+(defun yacas-in-comment-p ()
+  "Non-nil means that the point is in a comment."
+  (let ((ok t) result)
+    (save-excursion
+      (while ok
+	(cond ((search-backward "/" (- (point) 2000) t)
+	       (cond ((looking-at "/\\*")
+		      (setq result t ok nil))
+		     ((bobp) (setq ok nil ))
+		     ((progn (forward-char -1)
+			     (looking-at "\\*/"))
+		      (setq ok nil))))
+	      (t (setq ok nil))))
+      result)))
+
+(defun yacas-find-command-end ()
+  "Return the point that ends the current command"
+  (let ((out)
+        (mtch))
+    (save-excursion
+      (while (looking-at "[ \t\n]")
+        (forward-char 1))
+      (if (= (point) (point-max))
+          (setq out (point))
+        (re-search-forward "\\([[;\n]\\)\\|\\(/\\*\\)" nil t)      
+        (setq mtch (match-string 0))
+        (cond
+         ((string= mtch "/*")
+          (search-forward "*/")
+          (setq out (yacas-find-command-end)))
+         ((string= mtch "[")
+          (forward-char -1)
+          (forward-sexp)
+          (setq out (yacas-find-command-end)))
+         ((string= mtch ";")
+          (setq out (point)))
+         ((string= mtch "\n")
+          (forward-char -2)
+          (while (looking-at " ") (forward-char -1))
+          (if (looking-at "\\\\") 
+              (progn
+                (forward-line 1)
+                (setq out (yacas-find-command-end)))
+            (end-of-line)
+            (setq out (point)))))))
+    out))
+
+(defun yacas-find-command-end-string (string)
+  "return the point that ends the current command"
+  (let* ((tmpfile (make-temp-name "#ys"))
+         (tmpbuf (get-buffer-create tmpfile))
+         (out)
+         (mtch))
+    (save-excursion
+      (set-buffer tmpbuf)
+      (make-local-hook 'kill-buffer-hook)
+      (setq kill-buffer-hook nil)
+      (insert string)
+      (goto-char (point-min))
+      (setq out (yacas-find-command-end)))
+    (kill-buffer tmpbuf)
+;    out))
+    (- out 1)))
 
 (defun yacas-send-block (stuff)
   "Send a block of code to Yacas."
   (yacas-start)
-  (while (string-match ";" stuff)
-    (setq end (1+ (string-match ";" stuff)))
+  (while (not (string= "" stuff))
+    (setq end (yacas-find-command-end-string stuff))
+    ;    (setq end (1+ (string-match ";" stuff)))
     (yacas-single-command (substring stuff 0 end))
     (setq stuff (substring stuff end))))
 
+;;;;
+(defconst yacas-use-emacs-help-string-1
+   "Help():=SystemCall(\"emacsclient -e \\\"(browse-url \\\\\\\"\":FindFile(\"documentation/books.html\"):\"\\\\\\\")\\\"\");")
+
+(defconst yacas-use-emacs-help-string-2
+   "Help(_f)<--SystemCall(\"emacsclient -e \\\"(browse-url \\\\\\\"\":FindFile(\"documentation/ref.html\"):\"#\":f : \"\\\\\\\")\\\"\");")
+
+(defun yacas-use-emacs-help-start ()
+  (server-start)
+  (yacas-single-command yacas-use-emacs-help-string-1)
+  (yacas-single-command yacas-use-emacs-help-string-2))
+
 ;;; Sending information to the process should be done through these
 ;; next four commands
 
@@ -932,6 +1017,35 @@ do not display the yacas buffer."
     (yacas-string "PrettyPrinter(\"PrettyForm\");")
     (setq yacas-prettyform t)))
 
+;;; Get the output
+(defun yacas-output ()
+  "Get the most recent output from Yacas."
+  (interactive)
+  (save-excursion
+    (set-buffer (process-buffer inferior-yacas-process))
+    (let ((pmark (progn (goto-char (process-mark inferior-yacas-process))
+                        (forward-line 0)
+                        (point-marker))))
+      (buffer-substring-no-properties comint-last-input-end pmark))))
+
+(defun yacas-tex-output ()
+  "Get the most recent output from Yacas, assuming the output was in TeX form."
+  (interactive)
+  (save-excursion
+    (set-buffer (process-buffer inferior-yacas-process))
+    (let ((pmark (progn (goto-char (process-mark inferior-yacas-process))
+                        (forward-line 0)
+                        (point-marker)))
+          (beg)
+          (end))
+      (goto-char comint-last-input-end)
+      (search-forward "$")
+      (setq beg (point))
+      (goto-char pmark)
+      (search-backward "$")
+      (setq end (point))
+      (buffer-substring-no-properties beg end))))
+
 (add-hook 'yacas-start-shell-hooks 'yacas-toggle-prettyform)
 
 ;;; This menu part is stolen, more or less, from w3-menu.el, by William Perry
