@@ -2,17 +2,85 @@
 
 /*
 
+//    redraw(); //output changed
+//    Fl::flush();
+
 TODO:
+- proteus debugger, to step through code:
+  - text browser view
+  - debugging evaluator for fltk
+  - local variables
+  - call stack
+  - watch
+  - custom breakpoints
+- proteushints: also add documentation on proteus commands, so you get hints on them
+- dialogs for quickly adding certain types of commands to the note pad.
+- document the user interface
+- document the files that should be available, and where they should be.
+
+- "restart notepad" menu item
+- cursor: select part of input
+- select: copy/paste
+- hints, list current matches, and do auto-completion with selection filling out the rest.
+- graphing optimization: draw to bitmap, possible?
+- bug: putting current command on last line from a link.
+- Mac install: mac exes seem to care about where they are dumped.
+- fill out the help panes
+- Function finder dialog
+
+x not all functions get ripped correctly, see output from manripper
+x delete should do new hints too.
+x warnings for descriptions that don't have a calling convention (descriptions left in the map).
+x backup pixmap rendering.
+x manripper moved: also move in cvs
+
+
+- console drawer is slow: 
+  - render only what is on screen
+- menu bar resizes, i think it shouldn't
+- select, copy-paste
+- document graphing primitives
+- graphing cell:
+  - Why not load flplot by default?
+  - modify flplot so that all is local, so we can have multiple plots.
+  - if there is a ConsoleOutBase ready to be inserted,
+    insert it instead of the text returned.
+  - just for laughs, a formula pretty printer using the graphing capabilities.
+- function browser at the top, allowing you to insert, but also
+  showing a hint box when you hover over it, with a short description
+  of the command.
+- workspace, with easy access to different note pads, menu to the left or
+something.
+- re-instate the history list.
+- formula rendering/editing sub-cell.
+
+
+
+- input cells should maintain their font sizes, also when editing
+- you should be able to set currentFont size and type and color 
+  for the command line also.
+- document the graphing capabilities
+- prompt changing
+- command processing changing: $
+- graphing cell:
+  - flplot for new grapher, make an flplot2
+    - flplot2 should not be using any globals!
+  - consoleoutbase-derived class that does the same as Drawer, but in a
+cell.
+  - FltkGraphStart that adds a graphing cell, like the Proteus'AddCommand
+does.
+- flplot not installed?
+- flplot not loaded by default?
+
+- Notepad(FindFile("")) crashes, empty file name should be trapped
+  before fopen is called.
+- embedded grapher
+- embedded links, text
+- pp formulae
 
-x- if iLast==NULL, don't add texts
-x- set iLast to NULL in DeleteAll
-x- set iLast to NULL after a Notepad command
-x- remove all code relating to the old history list.
-x- remove and rethink the dirty scheme (don't respond to dirty right now)
 - if there is a selection,show the input line above the output, and as
   soon as the input line is modified, don't show the output. only show
   the output again when enter is pressed, or escape, etc.
-x- no resetting the cursor back to 0 when staying on the same line?
 - MakeSure.... should adjust to the top, not the bottom!
 
 
@@ -47,12 +115,18 @@ x- no resetting the cursor back to 0 when staying on the same line?
 #include <FL/Fl.H>
 #include <FL/fl_draw.H>
 #include <FL/Fl_Tabs.H>
+#include <FL/Fl_Scroll.H>
+#include <FL/Fl_Pixmap.H>
+#include <FL/Fl_Help_Dialog.H>
+
 #include "yacasprivate.h"
 #include "FltkConsole.h"
 #include "FltkHintWindow.h"
 #include "HelpView.h"
-#include "yacas.h"
 
+#ifdef WIN32
+#  define snprintf _snprintf
+#endif
 
 #define SUPPORT_NOTEPAD
 
@@ -64,10 +138,12 @@ const char* errorPrompt = "Error> ";
 const char* linkPrompt = "Link: ";
 #define BufSz 512
 
-#define SetInputFont() fl_font(FL_HELVETICA,iDefaultFontSize)
+#define SetInputFont() fl_font(iDefaultFontType,iDefaultFontSize)
 #define INPUT_PROMPT "$ "
 
 
+ConsoleOutBase* cell_to_insert = NULL;
+
 int LoadHints(char* file);
 void DisposeHints();
 
@@ -80,18 +156,27 @@ FltkConsole::~FltkConsole()
     DisposeHints();
 }
 
-FltkConsole::FltkConsole(int x, int y, int w, int h, int aDefaultFontSize)
-: Fl_Widget(x,y,w,h,NULL), iLast(NULL),iDefaultFontSize(aDefaultFontSize),
-hints(NULL),
-iOutputOffsetX(0), iOutputOffsetY(0),
+
+
+
+FltkConsole::FltkConsole(int x, int y, int w, int h)
+: Fl_Widget(x,y,w,h,NULL), iLast(NULL),hints(NULL),
+/*iOutputOffsetX(0), iOutputOffsetY(0), */
 iMouseDownX(0),iMouseDownY(0),iMovingOutput(0),
 iOutputHeight(0),iCurrentHighlighted(-1),
 iInputDirty(1),iOutputDirty(1),iShowInput(1),iEnableInput(1)
 {
+
+    ResetNotepadState();
+
     {
         extern char defdir[128];
         char buf[128];
-        sprintf(buf,"%shints",defdir);
+        extern int release_structure;
+        if (release_structure)
+          sprintf(buf,"%sdocumentation/hints",defdir);
+        else
+          sprintf(buf,"%smanmake/hints",defdir);
         {
           FILE*f=fopen(buf,"r");
           if (!f) 
@@ -102,7 +187,36 @@ iInputDirty(1),iOutputDirty(1),iShowInput(1),iEnableInput(1)
         }
         LoadHints(buf);
     }
+    
+    char fname[256];
+#ifdef HAVE_VSNPRINTF
+    snprintf(fname,256,"%s/.yacas_history",getenv("HOME"));
+#else
+    sprintf(fname,"%s/.yacas_history",getenv("HOME"));
+#endif
+    FILE*f=fopen(fname,"r");
+    if (f)
+    {
+        if(f)
+        {
+            char buff[BufSz];
+            while(fgets(buff,BufSz-2,f))
+            {
+                int i;
+                for(i=0;buff[i] && buff[i] != '\n';++i)
+                    ;
+                buff[i++] = '\0';
+                LispStringPtr ptr = NEW LispString(buff);
+                iHistoryList.Append(ptr);
+                
+            }
+            fclose(f);
+        }
+    }
+    iMaxHistoryLinesSaved = 1024;
+    iHistoryList.ResetHistoryPosition();
     CommandLineStartNew();
+    UpdateHeight(0);
 }
 
 void FltkConsole::DeleteAll()
@@ -114,13 +228,59 @@ void FltkConsole::DeleteAll()
         iConsoleOut.Delete(0);
     }
     iOutputHeight = 0;
+    UpdateHeight(0);
     iLast = NULL;
     iCurrentHighlighted = -1;
 }
 
 
+void FltkConsole::SaveNotePad(LispCharPtr aFile)
+{
+  FILE*f=fopen(aFile,"w");
+  if(f)
+  {
+    fprintf(f,"Proteus'AddLink(\"True\",\"Saved note pad: %s\",False,False);\n",aFile);
+    int i;
+    for (i=0;i<iConsoleOut.NrItems();i++)
+    {
+      iConsoleOut[i]->Save(f);
+    }
+    
+    fclose(f);
+  }
+}
+
+void FltkConsole::ResetNotepadState()
+{
+  iDefaultFontSize = 15;
+  iDefaultFontColor = FL_BLACK;
+  iDefaultFontType = FL_HELVETICA_BOLD;
+}
+
 void FltkConsole::LoadNotePad(LispCharPtr aFile)
 {
+  ResetNotepadState();
+
+  DeleteAll();
+  extern CYacas* yacas;
+  char buf[256];//TODO buffer overflow problem
+  snprintf(buf,256,"Load(\"%s\");",aFile);
+  yacas->Evaluate(buf);
+
+  extern LispString the_out;
+  the_out.SetNrItems(0);
+  the_out.Append('\0');
+
+  iOutputHeight+=15; // for the cursor, TODO what was the define again for the cursor height?
+  UpdateHeight(0);
+
+  SetCurrentHighlighted(0); // to update the initial line value
+  //        iOutputOffsetY = 0;
+  MakeSureHighlightedVisible();//
+  redraw(); //output changed
+  Fl::flush();
+  return;
+
     int prevshow = iShowInput;
     int prevenable = iEnableInput;
 
@@ -163,8 +323,6 @@ void FltkConsole::LoadNotePad(LispCharPtr aFile)
                 }
                 start++;
             }
-            
-
             if (!internal)
             {
                 AddGroup(showinput, enableinput);
@@ -213,10 +371,14 @@ void FltkConsole::LoadNotePad(LispCharPtr aFile)
             the_out.Append('\0');
         }
         fclose(f);
+
+        iOutputHeight+=15; // for the cursor, TODO what was the define again for the cursor height?
+        UpdateHeight(0);
+
         SetInputDirty();
         SetOutputDirty();
-        iCurrentHighlighted = 0;
-        iOutputOffsetY = 0;
+        SetCurrentHighlighted(0); // to update the initial line value
+//        iOutputOffsetY = 0;
         MakeSureHighlightedVisible();
         redraw();
     }
@@ -229,38 +391,37 @@ void FltkConsole::LoadNotePad(LispCharPtr aFile)
 void FltkConsole::CommandLineStartNew()
 {
     DeleteHints();
-    /*TODO remove?
-     history=iHistory.NrItems();
-     iHistoryUnchanged = 0;
-*/
     cursor=0;
 }
 
 void FltkConsole::SaveHistory()
 {
-    /*TODO remove?
-     char fname[256];
-    sprintf(fname,"%s/.yacas_history",getenv("HOME"));
-    
-    FILE*f=fopen(fname,"w");
-    if (f)
+  char fname[256];
+#ifdef HAVE_VSNPRINTF
+  snprintf(fname,256,"%s/.yacas_history",getenv("HOME"));
+#else
+  sprintf(fname,"%s/.yacas_history",getenv("HOME"));
+#endif
+  
+  FILE*f=fopen(fname,"w");
+  if (f)
+  {
+    int i;
+    int from=0;
+    if (iMaxHistoryLinesSaved>=0)
     {
-        int i;
-        int from=0;
-        if (iMaxLines>=0)
-        {
-            if (iHistory.NrItems()>iMaxLines)
-            {
-                from = iHistory.NrItems()-iMaxLines;
-            }
-        }
-        for (i=from;i<iHistory.NrItems();i++)
-        {
-            fprintf(f,"%s\n",iHistory[i]->String());
-        }
-        fclose(f);
-        }
-        */
+      if (iHistoryList.NrLines()>iMaxHistoryLinesSaved)
+      {
+        from = iHistoryList.NrLines()-iMaxHistoryLinesSaved;
+      }
+    }
+    for (i=from;i<iHistoryList.NrLines();i++)
+    {
+      LispStringPtr ptr = iHistoryList.GetLine(i);
+      fprintf(f,"%s\n",ptr->String());
+    }
+    fclose(f);
+  }
 }
 
 void FltkConsole::AddGroup()
@@ -278,6 +439,7 @@ void FltkConsole::AddText(LispCharPtr aText,int color, const char* aPrompt,
 {
     if (iLast == NULL)
         return;
+    iOutputHeight-=iLast->height();
     ConsoleOutBase* toadd;
 
     char buffer[BufSz+1];
@@ -297,7 +459,6 @@ void FltkConsole::AddText(LispCharPtr aText,int color, const char* aPrompt,
         {
             toadd = new ConsoleFlatText(aText, color,aPrompt,aFont,aFontSize);
             iLast->Add(toadd);
-            iOutputHeight+=toadd->height();
             aText+=len;
         }
         else
@@ -306,12 +467,13 @@ void FltkConsole::AddText(LispCharPtr aText,int color, const char* aPrompt,
             buffer[len] = '\0';
             toadd = new ConsoleFlatText(buffer, color,aPrompt,aFont,aFontSize);
             iLast->Add(toadd);
-            iOutputHeight+=toadd->height();
             if (aText[len] == '\n') aText++;
             aText+=len;
         }
     }
+    iOutputHeight+=iLast->height();
     iOutputDirty = 1;
+    UpdateHeight(0);
 }
 
 
@@ -319,8 +481,26 @@ void FltkConsole::CommandLineEnd()
 {
     DoLine(&iSubLine[0]);
 }
+
+void FltkConsole::Restart()
+{
+  DeleteAll();
+  {
+    extern LispPtr graph;
+    graph.Set(NULL);
+  }
+  
+  extern void RestartYacas();
+  RestartYacas();
+  extern Fl_Scroll *console_scroll;
+  console_scroll->redraw();
+}
+
 void FltkConsole::DoLine(char* inpline)
 {
+    int addable = 1;
+    int font_size = 0;
+
     if(*inpline)
     {
         extern Fl_Tabs* mainTabs;
@@ -339,7 +519,11 @@ void FltkConsole::DoLine(char* inpline)
                     char buf[120];
                     extern char defdir[128];
 
-                    sprintf(buf,"%s/documentation/ref.html#%s",defdir,&inpline[1]);
+                    extern int release_structure;
+                    if (release_structure)
+                      sprintf(buf,"%s/documentation/ref.html#%s",defdir,&inpline[1]);
+                    else
+                      sprintf(buf,"%s/manmake/ref.html#%s",defdir,&inpline[1]);
                     extern void HelpGo(char*);
                     HelpGo(buf);
                 }
@@ -351,66 +535,85 @@ void FltkConsole::DoLine(char* inpline)
         {
             if (!strncmp(inpline,"restart",7))
             {
-                DeleteAll();
-                {
-                    extern LispPtr graph;
-                    graph.Set(NULL);
-                }
-
-                extern void RestartYacas();
-                RestartYacas();
-                goto END;
+              Restart();
+              goto END;
             }
             else if (!strncmp(inpline,"quit",4))
             {
-                exit(0);
+              exit(0);
             }
         }
     }
 
+
+    extern CYacas* yacas;
+    yacas->Evaluate(inpline);
+
 #ifdef SUPPORT_NOTEPAD
     if (iCurrentHighlighted >= 0)
     {
-        if (iConsoleOut[iCurrentHighlighted]->IsEditable())
+        if (iConsoleOut[iCurrentHighlighted]->InputIsVisible())
         {
             iLast = (ConsoleGrouped*)iConsoleOut[iCurrentHighlighted];
             iOutputHeight -= iLast->height();
+            UpdateHeight(0);
+            font_size = iLast->Fontsize();
             iLast->DeleteAll();
         }
         else
-            AddGroup();
+        {
+          addable=0;
+//            AddGroup();
+        }
     }
     else
+    {
         AddGroup();
+    }
 #else
     AddGroup();
 #endif
-    AddText(inpline, FL_BLACK,inPrompt,FL_HELVETICA,iDefaultFontSize);
+
+    if (font_size==0) font_size=iDefaultFontSize;
+
+    if (addable)
+      AddText(inpline, iDefaultFontColor,inPrompt,iDefaultFontType,font_size);
     //SetInputDirty();
     SetOutputDirty();
-    redraw(); //output changed
-    Fl::flush();
+//    redraw(); //output changed
+//    Fl::flush();
     {
-        extern CYacas* yacas;
-        yacas->Evaluate(inpline);
+//        extern CYacas* yacas;
+//        yacas->Evaluate(inpline);
 
         extern LispString the_out;
         if (the_out[0])
         {
-            AddText(the_out.String(), FL_RED,printPrompt,FL_COURIER,iDefaultFontSize);
-            the_out.SetNrItems(0);
-            the_out.Append('\0');
+            if (addable)
+              AddText(the_out.String(), FL_RED,printPrompt,FL_COURIER,font_size);
             the_out.SetNrItems(0);
             the_out.Append('\0');
         }
 
         if (yacas->Error()[0] != '\0')
         {
-            AddText(yacas->Error(), FL_RED,errorPrompt,FL_HELVETICA,iDefaultFontSize);
+          if (addable)
+            AddText(yacas->Error(), FL_RED,errorPrompt,FL_HELVETICA,font_size);
         }
         else
         {
-            AddText(yacas->Result(), FL_BLUE,outPrompt,FL_HELVETICA,iDefaultFontSize);
+          if (addable)
+            AddText(yacas->Result(), FL_BLUE,outPrompt,FL_HELVETICA,font_size);
+        }
+        if (addable)
+        {
+          if (cell_to_insert)
+          {
+            iOutputHeight+=cell_to_insert->height();
+            iLast->Add(cell_to_insert);
+            cell_to_insert = NULL;
+            UpdateHeight(0);
+          }
         }
     }
 
@@ -447,13 +650,49 @@ void FltkConsole::SetInput(LispCharPtr aText, LispInt nr)
 }
 void FltkConsole::GetHistory(LispInt aLine)
 {
-    /*TODO remove?
-     SetInput(&(*iHistory[aLine])[0], iHistory[aLine]->NrItems());
-     */
 }
 
 void FltkConsole::MakeSureHighlightedVisible()
 {
+  extern Fl_Scroll *console_scroll;
+  if (iCurrentHighlighted < 0)
+  {
+    int newy;
+    newy=iOutputHeight+20-console_scroll->h();
+    if (newy<0) newy=0;
+    console_scroll->position(console_scroll->xposition(),newy);
+    SetOutputDirty();
+//    redraw(); //output changed
+//    Fl::flush();
+  }
+  else
+  {
+    int i,nr = iConsoleOut.NrItems();
+    int iy = 0;
+    for (i=0;i<nr && i<iCurrentHighlighted;i++)
+    {
+      int thisheight = iConsoleOut[i]->height();
+      iy+=thisheight;
+    }
+    extern Fl_Scroll *console_scroll;
+    int cons_h = console_scroll->h();
+    int hh=h();
+    int ypos = console_scroll->yposition();
+    int cellh = 0;
+    if (iConsoleOut.NrItems()>0)
+      cellh = iConsoleOut[iCurrentHighlighted]->height();
+    if (iy < ypos || iy+cellh > ypos+cons_h)
+    {
+      if (iy>iOutputHeight+20-console_scroll->h())
+        iy = iOutputHeight+20-console_scroll->h();
+      console_scroll->position(console_scroll->xposition(),iy);
+      SetOutputDirty();
+//      redraw(); //output changed
+//      Fl::flush();
+    }
+  }
+  return; //NEWA
+/*
 #ifdef SUPPORT_NOTEPAD
     if (iCurrentHighlighted < 0)
     {
@@ -498,6 +737,7 @@ void FltkConsole::MakeSureHighlightedVisible()
     }
 //printf("item %d, offs %d\n",iCurrentHighlighted,iOutputOffsetY);
 #endif
+*/
 }
 
 void FltkConsole::handle_key(int key)
@@ -507,64 +747,108 @@ void FltkConsole::handle_key(int key)
     case eDelete:
         if (cursor<iSubLine.NrItems()-1)
         {
-            iSubLine.Delete(cursor);
-            iFullLineDirty = 1;
-//TODO remove?            iHistoryUnchanged = 0;
+          if (iSubLine[cursor] == '(' || iSubLine[cursor] == ')')
+          {
+            delete hints;
+            hints=NULL;
+          }
+          iSubLine.Delete(cursor);
+          iFullLineDirty = 1;
+          if (hints == NULL)
+            hints = CheckForNewHints(&iSubLine[0],cursor);
         }
         break;
     case eBackSpace:
         if (cursor>0)
         {
-            cursor--;
-            iSubLine.Delete(cursor);
-            iFullLineDirty = 1;
-//TODO remove?            iHistoryUnchanged = 0;
+          cursor--;
+          if (iSubLine[cursor] == '(' || iSubLine[cursor] == ')')
+          {
+            delete hints;
+            hints=NULL;
+          }
+          iSubLine.Delete(cursor);
+          iFullLineDirty = 1;
+          if (hints == NULL)
+            hints = CheckForNewHints(&iSubLine[0],cursor);
         }
         break;
     case eLeft:
         if (cursor>0)
-            cursor--;
+        {
+          cursor--;
+
+          if (iSubLine[cursor] == '(' || iSubLine[cursor] == ')')
+          {
+            delete hints;
+            hints=NULL;
+          }
+          if (hints == NULL)
+            hints = CheckForNewHints(&iSubLine[0],cursor);
+        }
         break;
     case eRight:
         if (cursor<iSubLine.NrItems()-1)
-            cursor++;
+        {
+          if (iSubLine[cursor] == '(' || iSubLine[cursor] == ')')
+          {
+            delete hints;
+            hints=NULL;
+          }
+          cursor++;
+          if (hints == NULL)
+            hints = CheckForNewHints(&iSubLine[0],cursor);
+        }
         break;
     case eUp:
 
 #ifdef SUPPORT_NOTEPAD
         {
             DeleteHints();
+
+            if (Fl::event_shift())
+            {
+              if (iHistoryList.ArrowUp(iSubLine,cursor))
+              {
+                SetInputDirty();
+                redraw(); //output changed
+                Fl::flush();
+              }
+              break;
+            }
+
             if (iCurrentHighlighted>0)
-                SetCurrentHighlighted(iCurrentHighlighted-1);
+            {
+              SetCurrentHighlighted(iCurrentHighlighted-1);
+            }
             else if (iCurrentHighlighted == 0)
             {
             }
             else
-                SetCurrentHighlighted(iConsoleOut.NrItems()-1);
+              SetCurrentHighlighted(iConsoleOut.NrItems()-1);
             SetOutputDirty();
             MakeSureHighlightedVisible();
-            //                iOutputOffsetY += iConsoleOut[iCurrentHighlighted]->height();
-            //                iOutputOffsetY -= iConsoleOut[iCurrentHighlighted]->height();
             cursor = iSubLine.NrItems()-1;
+            redraw(); //output changed
+            Fl::flush();
             break;
         }
 #endif
-        
-        /*TODO remove?
-         if (history>0)
-        {
-            history--;
-            GetHistory(history);
-            cursor = iSubLine.NrItems()-1;
-            iFullLineDirty = 1;
-            iHistoryUnchanged = 1;
-            }
-            */
         break;
     case eDown:
 #ifdef SUPPORT_NOTEPAD
         {
             DeleteHints();
+
+            if (Fl::event_shift())
+            {
+              iHistoryList.ArrowDown(iSubLine,cursor);
+              SetInputDirty();
+              redraw(); //output changed
+              Fl::flush();
+              break;
+            }
+
             if (iCurrentHighlighted < 0)
             {
             }
@@ -574,56 +858,20 @@ void FltkConsole::handle_key(int key)
                 SetCurrentHighlighted(-1);
             SetOutputDirty();
             MakeSureHighlightedVisible();
+            redraw(); //output changed
+            Fl::flush();
             cursor = iSubLine.NrItems()-1;
             break;
         }
 #endif
-        /*TODO remove?
-        if (history<iHistory.NrItems()-1)
-        {
-            history++;
-            GetHistory(history);
-            cursor = iSubLine.NrItems()-1;
-            iFullLineDirty = 1;
-            iHistoryUnchanged = 1;
-        }
-        else if (history == iHistory.NrItems()-1)
-        {
-            iSubLine.SetNrItems(1);
-            iSubLine[0] = '\0';
-            cursor = iSubLine.NrItems()-1;
-            history++;
-            iFullLineDirty = 1;
-        }
-        */
         break;
     case eTab:
+        if (Fl::event_shift())
         {
-/*TODO remove?
-            LispInt prevhistory=history;
-            history = iHistory.NrItems()-1;
-            while (history>=0)
-            {
-                LispInt j=0;
-                while (j<iSubLine.NrItems()-1 &&
-                       j<iHistory[history]->NrItems())
-                {
-                    if (iSubLine[j] != (*iHistory[history])[j])
-                        goto CONTINUE;
-                    j++;
-                }
-
-                GetHistory(history);
-                cursor = iSubLine.NrItems()-1;
-                iFullLineDirty = 1;
-                iHistoryUnchanged = 1;
-                break;
-            CONTINUE:
-                history--;
-            }
-            if (history<0)
-                history = prevhistory;
-                */
+          iHistoryList.Complete(iSubLine,cursor);
+          redraw(); //output changed
+          Fl::flush();
+          break;
         }
         break;
     case eEscape:
@@ -631,8 +879,6 @@ void FltkConsole::handle_key(int key)
         iSubLine[0] = '\0';
         cursor = iSubLine.NrItems()-1;
         iFullLineDirty = 1;
-//TODO remove?        iHistoryUnchanged = 0;
-//TODO remove?        history=iHistory.NrItems();
         DeleteHints();
         iCurrentHighlighted = -1;
         iLast = NULL;
@@ -648,7 +894,7 @@ void FltkConsole::handle_key(int key)
         DeleteHints();
         if (iSubLine.NrItems()>1)
         {
-//TODO remove?            iHistory.Append(new LispString(iSubLine));
+            iHistoryList.AddLine(iSubLine);
             CommandLineEnd();
             if (iCurrentHighlighted < 0 )
                 CommandLineStartNew();
@@ -663,6 +909,7 @@ void FltkConsole::handle_key(int key)
                     free(text);
                 }
         }
+        iHistoryList.ResetHistoryPosition();
         return;
         break;
     default:
@@ -670,27 +917,16 @@ void FltkConsole::handle_key(int key)
             LispChar cc=(LispChar)key;
             iSubLine.Insert(cursor,cc);
             iFullLineDirty = 1;
-//TODO remove?            iHistoryUnchanged = 0;
-/*
-            if (hints == NULL)
+            if (cc == '(' || cc == ')')
             {
-                int ifrom,ito;
-                ifrom = cursor;
-                while (ifrom>0 && IsAlpha(iSubLine[ifrom])) ifrom--;
-                if (!IsAlpha(iSubLine[ifrom])) ifrom++;
-                if (ifrom<=cursor)
-                {
-                    ito = ifrom;
-                    while (ito<iSubLine.NrItems() && IsAlpha(iSubLine[ito])) ito++;
-                    if (ito>ifrom)
-                        TryToHint(ifrom,ito);
-                }
+              delete hints;
+              hints=NULL;
             }
-*/
+
         }
         cursor++;
         if (hints == NULL)
-          CheckForNewHints();
+          hints = CheckForNewHints(&iSubLine[0],cursor);
         break;
     }
 }
@@ -699,17 +935,10 @@ struct HintItem
 {
     char* base;
     char* hint;
+    char* description;
 };
 
-/*
- HintItem hintTexts[] =
-{
-    {"Integrate", "Integrate(var,from,to)expression" },
-    {"Sum","Sum(var,from,to,body)"},
-    {"Sum","Sum(list)"},
-    {"Taylor","Taylor(var,at,order)function"},
-};
- */
+
 CArrayGrower<HintItem> hintTexts;
 char* htex = NULL;
 int hoffsets[256];
@@ -752,10 +981,19 @@ int LoadHints(char* file)
                 hi.base = &htex[i];
                 while(htex[i] != ':') i++;
                 htex[i] = '\0';
+
+//printf("%s\n",hi.base);
+
                 i++;
                 hi.hint = &htex[i];
                 while(htex[i] != ':') i++;
                 htex[i] = '\0';
+
+                i++;
+                hi.description = &htex[i];
+                while(htex[i] != ':') i++;
+                htex[i] = '\0';
+
                 hintTexts.Append(hi);
             }
         }
@@ -781,51 +1019,59 @@ int LoadHints(char* file)
 }
 
 
-
-void FltkConsole::TryToHint(int ifrom,int ito)
+FltkHintWindow* FltkConsole::TryToHint(char* text, int length)
 {
-    int nrhints = hintTexts.NrItems();
-    int i,start;
-    start = hoffsets[(unsigned char)iSubLine[ifrom]];
-    if (start<0)
-        return;
-    for (i = start;i<nrhints;i++)
+  FltkHintWindow* hints = NULL;
+  int nrhints = hintTexts.NrItems();
+  int i,start;
+  start = hoffsets[(unsigned char)text[0]];
+  if (start<0)
+      return NULL;
+  for (i = start;i<nrhints;i++)
+  {
+    if ((unsigned char)text[0] < (unsigned char)hintTexts[i].base[0])
+      break;
+    int baselen = strlen(hintTexts[i].base);
+    if (length==baselen)
     {
-        if ((unsigned char)iSubLine[ifrom] < (unsigned char)hintTexts[i].base[0])
-            break;
-        if (!strncmp(&iSubLine[ifrom],hintTexts[i].base,strlen(hintTexts[i].base)))
-        {
-            if (hints == NULL)
-                CreateHints();
-            AddHintLine(hintTexts[i].hint);
-        }
+      if (!strncmp(text,hintTexts[i].base,baselen))
+      {
+        if (hints == NULL)
+            hints = CreateHints(iDefaultFontSize);
+        AddHintLine(hints, hintTexts[i].hint,hintTexts[i].description);
+      }
     }
+  }
+  return hints;
 }
 
-void FltkConsole::CheckForNewHints()
+
+FltkHintWindow* FltkConsole::CheckForNewHints(char* text, int length)
 {
+  FltkHintWindow* hints = NULL;
 	int braces = 1;
 
 	int ifrom,ito;
-	ito = cursor;
+	ito = length;
 
 	while (ito > 0 && braces > 0)
 	{
 	  ito --;
-	  if (iSubLine[ito] == '(')
+	  if (text[ito] == '(')
 		braces--;
-	  if (iSubLine[ito] == ')')
+	  if (text[ito] == ')')
 		braces++;
 	}
 	if (braces == 0 && ito>0)
 	{
 	  ifrom = ito-1;
-	  while (ifrom>0 && IsAlpha(iSubLine[ifrom])) ifrom--;
+	  while (ifrom>0 && IsAlpha(text[ifrom])) ifrom--;
 	  if (ifrom >= 0)
-		if (!IsAlpha(iSubLine[ifrom])) ifrom++;
+		if (!IsAlpha(text[ifrom])) ifrom++;
 	  if (ito>ifrom)
-		TryToHint(ifrom,ito);
+    hints = TryToHint(&text[ifrom],ito-ifrom);
 	}
+  return hints;
 }
 
 void FltkConsole::SetCurrentHighlighted(int i)
@@ -851,6 +1097,12 @@ void FltkConsole::SetCurrentHighlighted(int i)
 
 int FltkConsole::handle(int event)
 {
+    // ALT+key reserved for menu
+    if (Fl::event_alt())
+    {
+      return 0;
+    }
+
     //printf("event %d\n",event);
     switch (event)
     {
@@ -886,11 +1138,10 @@ int FltkConsole::handle(int event)
             }
             else
             {
-                if (event == FL_PUSH)
                 {
                     iMovingOutput = 1;
-                    iMoveBaseX = iOutputOffsetX;
-                    iMoveBaseY = iOutputOffsetY;
+                    iMoveBaseX = 0;
+                    iMoveBaseY = 0;
                     iMouseDownX = Fl::event_x();
                     iMouseDownY = Fl::event_y();
 
@@ -904,14 +1155,13 @@ int FltkConsole::handle(int event)
                         int i,nr = iConsoleOut.NrItems();
                         for (i=nr-1;i>=0;i--)
                         {
-                            if (lowy+iOutputOffsetY < iy)
+                            if (lowy+0/*iOutputOffsetY*/ < iy)
                                 break;
-                            if (lowy+iOutputOffsetY>iMouseDownY && lowy+iOutputOffsetY - iConsoleOut[i]->height()<iMouseDownY)
+                            if (lowy+0/*iOutputOffsetY*/>iMouseDownY && lowy+0/*iOutputOffsetY*/ - iConsoleOut[i]->height()<iMouseDownY)
                             {
                                 if (prevHighlight == i)
                                 {
                                     iCurrentHighlighted = i;
-                                    //TODO maybe?  handle_key(eEnter);
                                 }
                                 else
                                 {
@@ -927,20 +1177,6 @@ int FltkConsole::handle(int event)
                     }
 #endif
                 }
-                else
-                {
-                    iOutputOffsetX = iMoveBaseX + Fl::event_x() - iMouseDownX;
-                    iOutputOffsetY = iMoveBaseY + Fl::event_y()  - iMouseDownY;
-
-                    if (iOutputOffsetX > 0) iOutputOffsetX = 0;
-                    if (iOutputOffsetX < -600) iOutputOffsetX = -600;
-                    
-                    if (iOutputOffsetY < 0) iOutputOffsetY = 0;
-                    if (iOutputOffsetY > iOutputHeight) iOutputOffsetY = iOutputHeight;
-                    //SetInputDirty();
-                    SetOutputDirty();
-                    redraw(); //output changed
-                }
             }
         }
         break;
@@ -968,10 +1204,12 @@ int FltkConsole::handle(int event)
                     int delta = h()/4;
                     if (Fl::event_shift())
                     {
+/*
                          if (iOutputOffsetY+delta < iOutputHeight)
                             iOutputOffsetY += delta;
                         else
                             iOutputOffsetY = iOutputHeight;
+*/
                     }
                     else
                     {
@@ -984,10 +1222,12 @@ int FltkConsole::handle(int event)
                     int delta = h()/4;
                     if (Fl::event_shift())
                     {
+/*
                         if (iOutputOffsetY-delta > 0)
                             iOutputOffsetY -= delta;
                         else
                             iOutputOffsetY = 0;
+*/
                     }
                     else
                     {
@@ -1015,9 +1255,15 @@ void FltkConsole::InsertText(const LispCharPtr aText)
    {
     LispChar c = *aText++; 
     iSubLine.Insert(cursor++,c);
+    if (c == '(' || c == ')')
+    {
+      delete hints;
+      hints=NULL;
+    }
    }
+
    if (hints == NULL)
-     CheckForNewHints();
+      hints = CheckForNewHints(&iSubLine[0],cursor);
 }
 
 //iCurrentHighlighted
@@ -1049,23 +1295,23 @@ void FltkConsole::DrawInterEdit()
     int inputlowy = y()+h()-fl_height()-fl_descent();
     for (i=nr-1;i>=0;i--)
     {
-        if (lowy+iOutputOffsetY < iy)
+        if (lowy/*+iOutputOffsetY*/ < iy)
             break;
         int selected = (iCurrentHighlighted == i);
         int thisheight = iConsoleOut[i]->height(!selected);
 
         lowy = lowy - thisheight;
-        iConsoleOut[i]->draw(ix+iOutputOffsetX, lowy+iOutputOffsetY, iw,!selected);
+        iConsoleOut[i]->draw(ix/*+iOutputOffsetX*/, lowy/*+iOutputOffsetY*/, iw,!selected);
 #ifdef SUPPORT_NOTEPAD
         if (iCurrentHighlighted == i)
         {
             SetInputFont();
             lowy -= fl_height();
-            inputlowy = lowy+iOutputOffsetY;
-            DrawInputLine(lowy+iOutputOffsetY);
+            inputlowy = lowy/*+iOutputOffsetY*/;
+            DrawInputLine(lowy/*+iOutputOffsetY*/);
 
             fl_color(FL_BLACK);
-            fl_rect(ix+iOutputOffsetX, lowy+iOutputOffsetY, iw, thisheight+fl_height());
+            fl_rect(ix/*+iOutputOffsetX*/, lowy/*+iOutputOffsetY*/, iw, thisheight+fl_height());
         }
 #endif
     }
@@ -1081,10 +1327,13 @@ void FltkConsole::DrawInterEdit()
     iOutputDirty = 0;
     return;
 }
+
+
 void FltkConsole::DrawInputLine(int lowy)
 {
     const char* text = &iSubLine[0];
     int iix =  x() + (int)fl_width(INPUT_PROMPT);
+    SetInputFont();
     int cur = iix+(int)fl_width(text, cursor);
 
     int limit = x()+w()-8;
@@ -1094,7 +1343,6 @@ void FltkConsole::DrawInputLine(int lowy)
         cur = limit;
     }
 
-    SetInputFont();
     fl_color(FL_RED);
     fl_draw(INPUT_PROMPT,x(),lowy+fl_height()-fl_descent());
 
@@ -1127,16 +1375,16 @@ void FltkConsole::DrawUnderEdit()
         fl_clip(x(),y(),w(),h()-fl_height());
         for (i=nr-1;i>=0;i--)
         {
-            if (lowy+iOutputOffsetY < iy)
+            if (lowy/*+iOutputOffsetY*/ < iy)
                 break;
             int thisheight = iConsoleOut[i]->height();
             lowy = lowy - thisheight;
-            iConsoleOut[i]->draw(ix+iOutputOffsetX, lowy+iOutputOffsetY, iw);
+            iConsoleOut[i]->draw(ix/*+iOutputOffsetX*/, lowy/*+iOutputOffsetY*/, iw);
 #ifdef SUPPORT_NOTEPAD
             if (iCurrentHighlighted == i)
             {
                 fl_color(FL_BLACK);
-                fl_rect(ix+iOutputOffsetX, lowy+iOutputOffsetY, iw, thisheight);
+                fl_rect(ix/*+iOutputOffsetX*/, lowy/*+iOutputOffsetY*/, iw, thisheight);
             }
 #endif
         }
@@ -1150,96 +1398,27 @@ void FltkConsole::DrawUnderEdit()
     }
     iOutputDirty = 0;
     return;
-/*TODO remove?
-    fl_font(FL_HELVETICA,iDefaultFontSize);
-    fl_clip(x(),y(),w(),h());
-    fl_color(FL_WHITE);
-
-
-    int i,nr;
-    nr = iConsoleOut.NrItems();
-    int ix = x(),iy = y(), iw = w(), ih = h();
-    int lowy = iy+ih-fl_height();
-
-//    if (iInputDirty)
-    {
-        fl_rectf(x(),lowy,w(),fl_height());
-    }
-//    if (iOutputDirty)
-    {
-        fl_rectf(x(),y(),w(),ih-fl_height());
-    }
-
-    
-//    if (iInputDirty)
-    {
-        const char* text = &iSubLine[0];
-        int iix =  ix + (int)fl_width(INPUT_PROMPT);
-        int cur = iix+(int)fl_width(text, cursor);
-
-        int limit = x()+w()-8;
-        if (cur > limit)
-        {
-            iix += (limit - cur);
-            cur = limit;
-        }
-        
-        SetInputFont();
-        fl_color(FL_RED);
-        fl_draw(INPUT_PROMPT,ix,lowy+fl_height()-fl_descent());
-
-
-        fl_clip(ix + (int)fl_width(INPUT_PROMPT),lowy,w()-(int)fl_width(INPUT_PROMPT),fl_height());
-
-        fl_draw(text,iix,lowy+fl_height()-fl_descent());
-
-        fl_color(FL_BLACK);
-        fl_begin_line();
-        fl_vertex(cur,lowy);
-        fl_vertex(cur,lowy+fl_height());
-        fl_end_line();
-
-        fl_pop_clip();
-    }
-    iInputDirty = 0;
-    fl_pop_clip();
-
-
-//    if (iOutputDirty)
-    {
-        fl_clip(x(),y(),w(),h()-fl_height());
-        for (i=nr-1;i>=0;i--)
-        {
-            if (lowy+iOutputOffsetY < iy)
-                break;
-            int thisheight = iConsoleOut[i]->height();
-            lowy = lowy - thisheight;
-            iConsoleOut[i]->draw(ix+iOutputOffsetX, lowy+iOutputOffsetY, iw);
-#ifdef SUPPORT_NOTEPAD
-            if (iCurrentHighlighted == i)
-            {
-                fl_color(FL_BLACK);
-                fl_rect(ix+iOutputOffsetX, lowy+iOutputOffsetY, iw, thisheight);
-            }
-#endif
-        }
-        fl_pop_clip();
-
-        if (hints)
-        {
-            fl_font(FL_HELVETICA,iDefaultFontSize);
-            hints->draw(x(),y()+h()-fl_height()-fl_descent()-3);
-        }
-    }
-    iOutputDirty = 0;
-    return;
-*/
 }
 
+
+void FltkConsole::UpdateHeight(int aDelta)
+{
+  {
+    iOutputHeight+= aDelta;
+    extern Fl_Scroll *console_scroll;
+    if (iOutputHeight>0 && console_scroll)
+    {
+      resize(x(),y(),w(),iOutputHeight);
+      MakeSureHighlightedVisible();
+      console_scroll->redraw();
+    }
+  }
+}
 void FltkConsole::AddOutput(ConsoleOutBase* aOutput)
 {
-    iConsoleOut.Append(aOutput);
-    iOutputHeight+=aOutput->height();
+  iConsoleOut.Append(aOutput);
+  int extraHeight = aOutput->height();
+  UpdateHeight(extraHeight);
 }
 
 void FltkConsole::DeleteHints()
@@ -1252,21 +1431,32 @@ void FltkConsole::DeleteHints()
     }
 }
 
-void FltkConsole::CreateHints()
+FltkHintWindow* FltkConsole::CreateHints(int fontsize)
 {
-    DeleteHints();
-    hints = new FltkHintWindow(iDefaultFontSize);
-    iOutputDirty=1;
+  FltkHintWindow* hints;
+//    DeleteHints();
+  hints = new FltkHintWindow(fontsize);
+  iOutputDirty=1;
+  return hints;
 }
-void FltkConsole::AddHintLine(LispCharPtr aText)
+void FltkConsole::AddHintLine(FltkHintWindow* hints, LispCharPtr aText, LispCharPtr aDescription)
 {
     hints->AddLine(aText);
+    if (aDescription[0] != '\0')
+      hints->AddDescription(aDescription);
+
 }
 
+
 ConsoleOutBase::~ConsoleOutBase()
 {
 }
 
+void ConsoleFlatText::Save(FILE* f)
+{
+  fprintf(f,"Proteus'AddCommand(\"%s\");\n",iText.String());
+}
+
 ConsoleFlatText::ConsoleFlatText(LispCharPtr aText, int aColor, const char* aPrompt,
                                  int aFont,int aFontSize)
 {
@@ -1299,6 +1489,33 @@ LispCharPtr ConsoleFlatText::input()
 {
     return &iText[0];
 }
+
+int ConsoleFlatText::Fontsize() const
+{
+  return iFontSize;
+}
+
+
+void ConsoleGrouped::Save(FILE* f)
+{
+  if (iConsoleOut.NrItems()>0 && iShowInput && iEnableInput)
+  {
+    fprintf(f,":");
+    if (!iShowInput) fprintf(f,"i");
+    if (!iEnableInput) fprintf(f,"e");
+    fprintf(f,":");
+    iConsoleOut[0]->Save(f);
+  }
+}
+
+int ConsoleGrouped::Fontsize() const
+{
+  if (iConsoleOut.NrItems()>0)
+    return iConsoleOut[0]->Fontsize();
+  return 0;
+}
+
+
 LispCharPtr ConsoleGrouped::input()
 {
     if (!iEnableInput)
@@ -1375,3 +1592,61 @@ int ConsoleOutBase::InputIsVisible()
 {
     return 0;
 }
+
+
+
+ConsoleDrawer::ConsoleDrawer(LispEnvironment& aEnvironment,LispPtr& aExecute,int aWidth,int aHeight)
+: iEnvironment(aEnvironment),iExecute(aExecute),iWidth(aWidth),iHeight(aHeight), pixmap(NULL)
+{
+}
+
+ConsoleDrawer::~ConsoleDrawer()
+{
+  if (pixmap)
+    fl_delete_offscreen(pixmap);
+}
+void ConsoleDrawer::draw(int x, int y, int width,int draw_input)
+{
+  extern CYacas* yacas;
+  extern LispString the_out;
+  if (iExecute.Get())
+  {
+    if (!pixmap)
+    {
+      pixmap = (Fl_Offscreen)fl_create_offscreen(iWidth,iHeight);
+      fl_begin_offscreen(pixmap);
+  
+      fl_clip(0,0,iWidth,iHeight);
+  //    fl_translate(x,y);
+      LispPtr result;
+      iEnvironment.iEvaluator->Eval(iEnvironment,result,iExecute);
+  //    fl_translate(-x,-y);
+      fl_pop_clip();
+  
+      fl_end_offscreen();
+      //fl_draw_pixmap((const char*const*)p, 0,0,FL_BACKGROUND_COLOR);
+    }
+    if (pixmap)
+      fl_copy_offscreen(x,y,iWidth,iHeight,pixmap,0,0);
+  }
+}
+
+int ConsoleDrawer::height(int draw_input)
+{
+  return iHeight;
+}
+
+LispCharPtr ConsoleDrawer::input()
+{
+  return " ";
+}
+
+void ConsoleDrawer::Save(FILE* f)
+{
+}
+
+int ConsoleDrawer::Fontsize() const
+{
+  return 12;
+}
+
