@@ -810,23 +810,52 @@ void LispApplyPure(LispEnvironment& aEnvironment, LispInt aStackTop)
 }
 
 
+void LispPrettyReader(LispEnvironment& aEnvironment, LispInt aStackTop)
+{
+  LispInt nrArguments = InternalListLength(ARGUMENT(0));
+
+  if (nrArguments == 1)
+  {
+    aEnvironment.SetPrettyReader(NULL);
+  }
+  else
+  {
+    CHK_CORE(nrArguments == 2,KLispErrWrongNumberOfArgs);
+    LispPtr oper;
+    oper.Set(ARGUMENT(0).Get());
+    oper.GoNext();
+    CHK_ISSTRING_CORE(oper,1);
+    aEnvironment.SetPrettyReader(oper.Get()->String());
+  }
+  InternalTrue(aEnvironment,RESULT);
+}
+
+void LispGetPrettyReader(LispEnvironment& aEnvironment, LispInt aStackTop)
+{
+  if (aEnvironment.PrettyReader() == NULL)
+    RESULT.Set(LispAtom::New(aEnvironment,"\"\""));
+  else
+    RESULT.Set(LispAtom::New(aEnvironment,aEnvironment.PrettyReader()->String()));
+}
+
 void LispPrettyPrinter(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
-    LispInt nrArguments = InternalListLength(ARGUMENT(0));
+  LispInt nrArguments = InternalListLength(ARGUMENT(0));
 
-    if (nrArguments == 1)
-    {
-        aEnvironment.SetPrettyPrinter(NULL);
-    }
-    else
-    {
-        CHK_CORE(nrArguments == 2,KLispErrWrongNumberOfArgs);
-        LispPtr oper;
-        oper.Set(Argument(ARGUMENT(0),1).Get());
-        CHK_ISSTRING_CORE(oper,1);
-        aEnvironment.SetPrettyPrinter(oper.Get()->String());
-    }
-    InternalTrue(aEnvironment,RESULT);
+  if (nrArguments == 1)
+  {
+    aEnvironment.SetPrettyPrinter(NULL);
+  }
+  else
+  {
+    CHK_CORE(nrArguments == 2,KLispErrWrongNumberOfArgs);
+    LispPtr oper;
+    oper.Set(ARGUMENT(0).Get());
+    oper.GoNext();
+    CHK_ISSTRING_CORE(oper,1);
+    aEnvironment.SetPrettyPrinter(oper.Get()->String());
+  }
+  InternalTrue(aEnvironment,RESULT);
 }
 
 void LispGetPrettyPrinter(LispEnvironment& aEnvironment, LispInt aStackTop)
@@ -837,8 +866,6 @@ void LispGetPrettyPrinter(LispEnvironment& aEnvironment, LispInt aStackTop)
     RESULT.Set(LispAtom::New(aEnvironment,aEnvironment.PrettyPrinter()->String()));
 }
 
-
-
 void LispGarbageCollect(LispEnvironment& aEnvironment, LispInt aStackTop)
 {
     //TESTARGS(1);
@@ -921,20 +948,23 @@ void LispDllLoad(LispEnvironment& aEnvironment, LispInt aStackTop)
 
   LispString oper;
   InternalUnstringify(oper, string);
-  LispDllBase *dll;
+  LispDllBase *dll=NULL;
   LispInt opened = LispFalse;
 
   ExePluginMaker maker = FindExePlugin(&oper[0]);
   if (maker != NULL)
   {
     dll  = NEW ExeDll(maker);
-    Check(dll != NULL,KLispErrNotEnoughMemory);
   }
+#ifndef DISABLE_DYNAMIC
   else
   {
     dll  = NEW DLLCLASS;
-    Check(dll != NULL,KLispErrNotEnoughMemory);
   }
+#endif // DISABLE_DYNAMIC
+  Check(dll != NULL,KLispErrNotEnoughMemory);
+
+
 #ifdef YACAS_DEBUG
   printf("DLL allocated\n");
 #endif// YACAS_DEBUG
