@@ -42,7 +42,11 @@
 #define LispTrue 1
 
 // Dynamic link library loader class
-#define DLLCLASS    ElfDll
+#ifndef PLUGIN_HANDLER
+  #define DLLCLASS    LtDll // LispDllBase // ElfDll
+#else
+  #define DLLCLASS    LispDllBase 
+#endif
 
 // System calls
 #define SystemCall system
@@ -63,26 +67,29 @@
     } \
     catch(LispInt b) \
     { \
-     if (_e.iInputStatus.LineNumber()>=0) \
-        { \
-            LispChar linenum[20]; \
-            InternalIntToAscii(linenum,_e.iInputStatus.LineNumber()); \
-            _o.Write(_e.iInputStatus.FileName());\
-            _o.Write("(");\
-            _o.Write(linenum);\
-            _o.Write(") : ");\
+     if (_e.ErrorString(b)[0] != '\0') \
+     {\
+      if (_e.iInputStatus.LineNumber()>=0) \
+      { \
+          LispChar linenum[20]; \
+          InternalIntToAscii(linenum,_e.iInputStatus.LineNumber()); \
+          _o.Write(_e.iInputStatus.FileName());\
+          _o.Write("(");\
+          _o.Write(linenum);\
+          _o.Write(") : ");\
         } \
         _e.iCleanup.Delete(); \
         _o.Write(_e.ErrorString(b)); \
         _o.Write("\n"); \
+      }\
     }
 
-//            CHECKPTR(_e.iInputStatus.FileName());\
-//            _o.Write("Error on line ");\
-//            _o.Write(linenum);\
-//            _o.Write(" in file [");\
-//            _o.Write(_e.iInputStatus.FileName());\
-//            _o.Write("]\n"); \
+//            CHECKPTR(_e.iInputStatus.FileName());
+//            _o.Write("Error on line ");
+//            _o.Write(linenum);
+//            _o.Write(" in file [");
+//            _o.Write(_e.iInputStatus.FileName());
+//            _o.Write("]\n"); 
 
 
 // The following line should also be printed. BUT: it causes
@@ -109,11 +116,16 @@
 #define PlatSignedDoubleWord signed short
 */
 
-/*TODO*/
-#define PlatWord unsigned short
-#define PlatDoubleWord unsigned long
-#define PlatSignedDoubleWord signed long
-/**/
+//#define SUPPORT_LONG_LONG
+#ifdef  SUPPORT_LONG_LONG
+#  define PlatWord unsigned long
+#  define PlatDoubleWord unsigned long long
+#  define PlatSignedDoubleWord signed long long
+#else
+#  define PlatWord unsigned short
+#  define PlatDoubleWord unsigned long
+#  define PlatSignedDoubleWord signed long
+#endif
 
 
 
